@@ -1,16 +1,15 @@
-mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
+mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
                    alt.subset = NULL, reflevel= NULL,
                    nests = NULL, un.nest.el = FALSE, unscaled = FALSE,
-                   heterosc = FALSE, rpar = NULL,
+                   heterosc = FALSE, rpar = NULL, probit = FALSE,
                    R = 40, correlation = FALSE, halton = NULL, random.nb = NULL,
-                   panel = FALSE, estimate = TRUE, ...){
+                   panel = FALSE, estimate = TRUE, seed = 10, ...){
 
   start.time <- proc.time()
   callT <- match.call(expand.dots = TRUE)
   callF <- match.call(expand.dots = FALSE)
   formula <- callF$formula <- mFormula(formula)
   nframe <- length(sys.calls())
-
   heterosc.logit <- heterosc
   nested.logit <- !is.null(nests)
   if (!is.null(nests) && length(nests) == 1 && nests == "pcl"){
@@ -20,14 +19,9 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
   else pair.comb.logit <- FALSE
   if (nested.logit) attr(nests, "unique") <- ifelse(un.nest.el, TRUE, FALSE)
   mixed.logit <- !is.null(rpar)
-  multinom.logit <- !heterosc & is.null(nests) & is.null(rpar) 
-
-  if (heterosc.logit + nested.logit + mixed.logit > 1)
-    stop("only one of heterosc, rpar and nests can be used")
-
-##   if (!multinom.logit && !is.null(callT$method) && callT$method == 'nr')
-##     stop("nr method only implemented for the simple multinomial logit mode")
-
+  multinom.logit <- !heterosc & is.null(nests) & is.null(rpar) & !probit
+  if (heterosc.logit + nested.logit + mixed.logit + probit > 1)
+    stop("only one of heterosc, rpar, nests and probit can be used")
   if (multinom.logit){
     if (is.null(callT$method)) callT$method <- 'nr'
     if (is.null(callT$print.level)) callT$print.level <- 0
@@ -37,21 +31,21 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
   #  check whether arguments for mlogit.data are present: if so run
   #  mlogit.data
   ################################################################
+
   mldata <- callT
   response.name <- paste(deparse(attr(formula, "lhs")[[1]]))
   m <- match(c("data", "choice", "shape", "varying", "sep",
                "alt.var", "chid.var", "alt.levels",
-               "opposite", "drop.index", "id"),
+               "opposite", "drop.index", "id", "ranked"),
              names(mldata), 0L)
   use.mlogit.data <- sum(m[-1]) > 0
   if (use.mlogit.data){
     mldata <- mldata[c(1L, m)]
     mldata[[1L]] <- as.name("mlogit.data")
     mldata$choice <- response.name
-#    data <- eval(mldata, parent.frame())
     data <- eval(mldata, sys.frame(which = nframe))
   }
-  
+
   # 2 ###########################################################
   # model.frame (with the data provided or the one computed by
   # mlogit.data
@@ -61,25 +55,18 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
   m <- match(c("formula", "data", "subset", "na.action", "weights"),
              names(mf), 0L)
   mf <- mf[c(1L, m)]
-  mf$drop.unused.levels <- TRUE
+  # Est-ce important ??
+#  mf$drop.unused.levels <- TRUE
   mf$formula <- formula
   mf[[1L]] <- as.name("model.frame")
-  if (use.mlogit.data) mf$data <- data
-  # if the user called the data.frame "mldata", this conflicts with
-  # the call. The following line seems to fix the bug
-  mf$data <- data
+  mf$data <- data # fix the bug when the data is called mldata
   mf <- eval(mf, sys.frame(which = nframe))
-  
+
   # change the reference level of the response if required
-  if (!is.null(reflevel)){
-    attr(mf, "index")[["alt"]] <-
-      relevel(attr(mf, "index")[["alt"]], reflevel)
-  }
+  if (!is.null(reflevel)) attr(mf, "index")[["alt"]] <- relevel(attr(mf, "index")[["alt"]], reflevel)
   index <- attr(mf, "index")
   alt <- index[["alt"]]
   chid <- index[["chid"]]
-  alt.lev <- levels(alt)
-
   if (panel){
     if (!mixed.logit) stop("panel is only relevant for mixed logit models")
     id <- index[["id"]]
@@ -106,10 +93,13 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
   # alternative is not relevant
   alt.un <- unique(alt)
   chid.un <- unique(chid)
+  alt.lev <- levels(alt)
+  J <- length(alt.lev)
   n <- length(chid.un)
   T <- length(alt.un)
   balanced <- TRUE
   if (nrow(mf) != (n * T)){
+    # la ligne ci-dessous ne sert à rien ?
     rownames(mf) <- paste(chid, alt, sep = ".")
     all.rn <- as.character(t(outer(chid.un, alt.un, paste, sep = ".")))
     mf <- mf[all.rn, ]
@@ -140,7 +130,6 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
   choice <- na.omit(alt[y])
   X <- model.matrix(formula, mf)
   K <- ncol(X)
-  n <- nrow(X)
   df.residual <- n - K
   colnamesX <- colnames(X)
   if (any(names(mf)=="(weights)")){
@@ -149,33 +138,41 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
   }
   else weights <- NULL
   freq <- table(alt[y])
-  otime <- proc.time()
+  # Xl and yl are lists of length J which contains n matrix / vector
+  # of covariates and response (a boolean) ; yv is a vector that
+  # contains the chosen alternative
+  Xl <- vector(length = J, mode = "list")
+  names(Xl) <- levels(alt)
+  for (i in levels(alt)){
+    Xl[[i]] <- X[alt == i, , drop = FALSE]
+  }
+  yl <- split(y, alt)
+  yl <- lapply(yl, function(x){x[is.na(x)] <- FALSE ; x})
 
-  X <- split(as.data.frame(X), alt)
-  X <- lapply(X, as.matrix)
-  y <- split(y, alt)
-  y <- lapply(y, function(x){x[is.na(x)] <- FALSE ; x})
   
-  if (mixed.logit){
-    # if some random parameters are linked to factors, change the name
-    # to the second level (only relevant for 2 levels factors (to be
-    # tested)
-    for (i in 1:length(rpar)){
-      n <- names(rpar)[i]
-      clvar <- class(data[[n]])
-      if (inherits(clvar, "factor")){
-        lvar <- levels(data[[n]])[2]
-        names(rpar)[i] <- paste(n, lvar, sep="")
+  if (probit){
+    # for probit the response is a vector that contains the chosen
+    # alternative as a numeric ; NA values of y are replaced by FALSE
+    # so that the chosen alternative is correctly returned
+    y2 <- y
+    y2[is.na(y2)] <- FALSE
+    yv <- as.numeric(alt[y2])
+    # DX is a list of covariates first differenced with respect with the
+    # chosen alternative
+    DX <- vector("list", length = J - 1)
+    DX <- lapply(DX, function(x) return(matrix(NA, nrow = n, ncol = K)))
+    
+    for (i in 1:n){
+      any <- yv[i]
+      j <- 1
+      newj <- 1
+      for (j in 1:J){
+        if (j != any){
+          DX[[newj]][i,] <- Xl[[j]][i, ] - Xl[[any]][i,]
+          newj <- newj + 1
+        }
       }
     }
-    Vara <- sort(match(names(rpar), colnames(X[[1]])))
-    Varc <- (1:K)[-Vara]
-    Ka <- length(Vara)
-    Xa <- lapply(X, function(x) x[, Vara, drop = F])
-    Xc <- lapply(X, function(x) x[, Varc, drop = F])
-    # create the random numbers matrix
-    if (is.null(random.nb)) random.nb <- make.random.nb(R, Ka, halton)
-    colnames(random.nb) <- colnames(Xa[[1]])
   }
 
   # 4 ###########################################################
@@ -184,21 +181,18 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
 
   # first give names to the supplementary coefficients and values if
   # start is null
-
   if (nested.logit){
-    J <- length(nests)
+    L <- length(nests)
     if (un.nest.el){
       if (is.null(start) || length(start) == K) sup.coef <- c(iv = 1)
       names.sup.coef <- 'iv'
     }
     else{
-      if (is.null(start) || length(start) == K) sup.coef <- rep(1, J)
+      if (is.null(start) || length(start) == K) sup.coef <- rep(1, L)
       names.sup.coef <- paste("iv", names(nests), sep = ".")
     }
   }
   if (pair.comb.logit){
-    unalt <- levels(alt)
-    J <- length(unalt)
     if (un.nest.el){
       if (is.null(start)) sup.coef <- c(iv = 1)
       names.sup.coef <- 'iv'
@@ -206,7 +200,7 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
     else{
       names.sup.coef <- NULL
       for (i in 1:(J-1)){
-        names.sup.coef <- c(names.sup.coef, paste('iv', unalt[i], unalt[(i+1):J], sep = "_"))
+        names.sup.coef <- c(names.sup.coef, paste('iv', alt.lev[i], alt.lev[(i+1):J], sep = "."))
       }
       sup.coef <- rep(1, length(names.sup.coef))
       # in case we have to suppress one nest
@@ -215,17 +209,17 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
     }
   }
   if (heterosc.logit){
-    unalt <- levels(alt)
-    J <- length(unalt)
     if (is.null(start) || length(start) == K) sup.coef <- rep(1, J-1)
-    names.sup.coef <- paste("sp", unalt[-1], sep = ".")
+    names.sup.coef <- paste("sp", alt.lev[-1], sep = ".")
   }
   if (mixed.logit){
+    Vara <- sort(match(names(rpar), colnamesX))
+    Varc <- (1:K)[- Vara]
     nmean <- length(c(Varc, Vara))
     nvar <- length(Vara)
     if (!correlation){
       if (is.null(start) || length(start) == K) sup.coef <- rep(.1, nvar)
-      names.sup.coef <- paste("sd", colnames(Xa[[1]]), sep = ".")
+      names.sup.coef <- paste("sd", colnamesX[Vara], sep = ".")
     }
     else{
       if (is.null(start) || length(start) == K) sup.coef <- rep(.1, 0.5 * nvar * (nvar + 1))
@@ -237,29 +231,35 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
       }
     }
   }
-  
-## if no starting values are provided, provide a 0 vector and, if the
-## model is not the multinomial logit, estimate the model
-##   start can be :
-##   1. NULL, in this case estimate the multinomial logit model,
-##   2. a vector of lengthK ; then add starting values for the
-##   supplementary coefficients,
-##   3. a full set ; then just name the coefs.
-
+  if (probit){
+    names.sup.coef <- c()
+    for (i in 2:J){
+      names.sup.coef <- c(names.sup.coef,
+                          paste(alt.lev[i], alt.lev[i:J], sep = "."))
+    }
+    if (is.null(start) || length(start) == K){
+      corrMat <- matrix(0.5, J - 1, J - 1)
+      diag(corrMat) <- 1
+      corrMat <- chol(corrMat)
+      sup.coef <- (t(corrMat)[!upper.tri(corrMat)])
+      names(sup.coef) <- names.sup.coef
+    }
+  }
+  ## start can be :
+  ##   1. NULL, in this case estimate the multinomial logit model,
+  ##   2. a vector of length K ; then add starting values for the
+  ##   supplementary coefficients,
+  ##   3. a full set ; then just name the coefs.
   if (is.null(start)){
     callst <- callT
     start <- rep(0, K)
-    names(start) <- colnames(X[[1]])
+    names(start) <- colnamesX
     callst$start <- start
     callst$print.level <- 0
     if (!multinom.logit){
-      callst$nests <- NULL
-      callst$heterosc <- FALSE
-      callst$rpar <- NULL
-      callst$panel <- FALSE
-      callst$constPar <- NULL
-      callst$iterlim <- NULL
-      callst$print.level = 0
+      callst$nests <- callst$rpar <- callst$constPar <- callst$iterlim <- NULL
+      callst$heterosc <- callst$panel <- callst$probit <- FALSE
+      callst$print.level <- 0
       start <- coef(eval(callst, sys.frame(which=nframe)))
       if (mixed.logit){
         ln <- names(rpar[rpar == "ln"])
@@ -268,40 +268,63 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
     }
   }
   if (length(start) == K){
-    names(start) <- colnames(X[[1]])
+    names(start) <- colnamesX
     if (!multinom.logit){
       names(sup.coef) <- names.sup.coef
+      if (probit) start <- start * sqrt(3) / pi
       start <- c(start, sup.coef)
     }
   }
   else{
-    if (!multinom.logit) names(start) <- c(colnames(X[[1]]), names.sup.coef)
-    else names(start) <- colnames(X[[1]])
+    if (!multinom.logit) names(start) <- c(colnamesX, names.sup.coef)
+    else names(start) <- colnamesX
   }
 
+  # if constPar is numeric, insert the relevant value in the start
+  # vector and transform the constPar vector to character
+
   # 5 ###################################################################
   # Estimate the model using mlogit.nlm and passing the correct arguments
   #######################################################################
 
   opt <- callT
+  if (!is.null(opt$constPar)){
+    theconstPar <- eval(opt$constPar)
+    if (is.numeric(theconstPar)){
+      if (is.null(names(theconstPar))) stop('the numeric constPar vector should be named')
+      start[names(theconstPar)] <- theconstPar
+      opt$constPar <- names(theconstPar)
+    }
+  }
+  if (probit) if (is.null(opt$constPar)) opt$constPar <- names.sup.coef[1]
   opt$start <- start
   m <- match(c("method", "print.level", "iterlim",
                "start", "constPar","tol", "ftol", "steptol"),
              names(opt), 0L)
   opt <- opt[c(1L, m)]
   opt[[1]] <- as.name('mlogit.optim')
-  opt$logLik <- as.name('lnl.mlogits')
-  if (!mixed.logit) opt[c('X', 'y')] <- list(as.name('X'), as.name('y'))
+  opt$logLik <- as.name('lnl.slogit')
+
+  opposite <- TRUE
+  if (is.null(weights)) weights <- 1
+  opposite <- ifelse(opposite, -1, +1)
+  opt[c('weights', 'opposite')] <- list(as.name('weights'), as.name('opposite'))
+  if (!probit) opt[c('X', 'y')] <- list(as.name('Xl'), as.name('yl'))
+  else opt[c('X', 'y')] <- list(as.name('DX'), as.name('yv'))
   if (mixed.logit){
     opt$logLik <- as.name('lnl.rlogit')
-    opt[c('Xa', 'Xc', 'y', 'Varc', 'Vara', 'random.nb', 'id', 'rpar', 'correlation')] <-
-    list(as.name('Xa'), as.name('Xc'), as.name('y'), as.name('Varc'), as.name('Vara'),
-         as.name('random.nb'), as.name('id'), as.name('rpar'), as.name('correlation'))
+    opt[c('R', 'seed', 'id', 'rpar', 'correlation')] <-
+      list(as.name('R'), as.name('seed'), as.name('id'), as.name('rpar'), as.name('correlation'))
+  }
+  if (probit){
+    opt$logLik <- as.name('lnl.mprobit')
+    opt[c('R', 'seed')] <-
+      list(as.name('R'), as.name('seed'))
   }
   if (heterosc.logit){
     opt$logLik <- as.name('lnl.hlogit')
     rn <- gauss.quad(R, kind = "laguerre")
-    opt[c('rn', 'choice')] <- list(as.name('rn'), as.name('choice'))
+    opt[c('rn')] <- list(as.name('rn'))
   }
   if (nested.logit){
     opt$logLik <- as.name('lnl.nlogit')
@@ -311,90 +334,158 @@ mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
   }
   if (pair.comb.logit){
     opt$logLik <- as.name('lnl.nlogit')
-    alt.lev <- levels(alt)
-    J <- length(alt.lev)
     alt1 <- rep(alt.lev, c((J-1):0))
     alt2 <- alt.lev[unlist(lapply(2:J, function(x) x:J))]
-    names.nests <- paste(alt1, alt2, sep = "")
-    lnests <- mapply(function(x,y) c(x,y), alt1, alt2, SIMPLIFY = FALSE)
-    names(lnests) <- names.nests
+    names.nests <- paste(alt1, alt2, sep = ".")
+    nests <- mapply(function(x,y) c(x,y), alt1, alt2, SIMPLIFY = FALSE)
+    names(nests) <- names.nests
     # in case we have to suppress one nest
     # lnests <- lnests[-1]
-    opt$nests <- lnests
+    opt$nests <- nests
     opt$unscaled <- as.name('unscaled')
     opt$un.nest.el <- as.name('un.nest.el')
   }
-  if (!is.null(weights)) opt$weights <- as.name('weights')
-  opt$opposite <- TRUE
-
   x <- eval(opt, sys.frame(which = nframe))
 
   # 6 ###########################################################
   # put the result in form
   ###############################################################
 
+  # some general features
   n <- sum(freq)
+  x$est.stat$elaps.time <- proc.time() - start.time
   logLik <- structure( - as.numeric(x$optimum),
                       df = length(x$coefficients),
                       null = sum(freq * log(freq / n)),
                       class = "logLik"
                       )
-  ## logLik <- - as.numeric(x$optimum)
-  ## attr(logLik,"df") <- length(x$coefficients)
-  ## attr(logLik, 'null') <- sum(freq*log(freq/n))
-  ## class(logLik) <- "logLik"
-  fit <- as.matrix(data.frame(attr(x$optimum, 'probabilities')))
+  if (mixed.logit) rpar <- make.rpar(rpar, correlation, x$coefficients, NULL) else rpar <- NULL
+  if (!(nested.logit | pair.comb.logit)) nests <- NULL
   
-  if (heterosc){
-    opt$gradient <- FALSE
-    opt$logLik <- opt$iterlim <- opt$method <- opt$print.level <- opt$tol <- opt$constPar <- NULL
-#    opt['logLik', 'iterlim', 'method', 'print.level', 'tol', 'constPar'] <- NULL
-    opt[[1]] <- as.name('lnl.hlogit')
-    names(opt)[[2]] <- 'param'
-    fit <- c()
-    for (j in 1:J){
-      they <- vector(mode='list', length= J)
-      they <- lapply(they, function(x) rep(FALSE, n))
-      they[[j]] <- rep(TRUE, n)
-      opt$y <- they
-      fit <- cbind(fit,
-                   attr(eval(opt, sys.frame(which = nframe)), 'probabilities'))
-    }
-    colnames(fit) <- alt.lev
-  }
-  ## if (!nested.logit)  resid <- as.matrix(data.frame(y)) - fit
-  ## else resid <- NULL
-  resid <- as.matrix(data.frame(y)) - fit
-    
   # if no hessian is returned, use the BHHH approximation
-  if (is.null(attr(x$optimum, 'hessian')))
-    hessian <- - crossprod(attr(x$optimum, 'gradi'))
+  if (is.null(attr(x$optimum, 'hessian'))) hessian <- - crossprod(attr(x$optimum, 'gradi'))
   else hessian <- - attr(x$optimum, 'hessian')
-  x$est.stat$elaps.time <- proc.time() - start.time
+  fitted <- attr(x$optimum, "fitted")
+  probabilities <- attr(x$optimum, "probabilities")
+  resid <- Reduce("cbind", yl) - fitted
+  attr(x$coefficients, "fixed") <- attr(x$optimum, "fixed")
+  gradient <- -  attr(x$optimum, "gradient")
 
-  if (mixed.logit)   rpar <- make.rpar(rpar, correlation, x$coefficients, NULL)
-  else rpar <- NULL
+  
+  # compute the probabilities for all the alternatives for
+  # heteroscedastic and the probit model
+  if (probit | heterosc){
+    opt$logLik <- opt$iterlim <- opt$method <- opt$print.level <- opt$tol <- opt$constPar <- NULL
+    names(opt)[[2]] <- 'param'
+    opt[[2]] <- x$coefficients
+    opt$gradient <- FALSE
+    opt[[1]] <- as.name('lnl.hlogit')
+    if (probit) opt[[1]] <- as.name('lnl.mprobit') else opt[[1]] <- as.name('lnl.hlogit')
+    probabilities <- c()
+    for (k in 1:J){
+      if (probit){
+        they <- rep(k, n)
+        opt$y <- they
+        DX <- vector("list", length = J - 1)
+        DX <- lapply(DX, function(x) return(matrix(NA, nrow = n, ncol = K)))
+        for (i in 1:n){
+          any <- k
+          j <- 1
+          newj <- 1
+          for (j in 1:J){
+            if (j != any){
+              DX[[newj]][i, ] <- Xl[[j]][i, ] - Xl[[any]][i, ]
+              newj <- newj + 1
+            }
+          }
+        }
+        opt$X <- DX
+      }
+      if (heterosc){
+        they <- vector(mode='list', length= J)
+        they <- lapply(they, function(x) rep(FALSE, n))
+        they[[k]] <- rep(TRUE, n)
+        names(they) <- names(yl)
+        opt$y <- they
+      }
+      probabilities <- cbind(probabilities,
+                             attr(eval(opt, sys.frame(which = nframe)), 'fitted'))
+    }
+    colnames(probabilities) <- alt.lev
+    attr(x$optimum, "probabilities") <- probabilities
+  }
+
+  # Compute the covariance matrix of the errors
+  if (multinom.logit | mixed.logit){
+    alt.names <- colnames(probabilities)
+    J <- length(alt.names)
+    Omega <- matrix(0, J, J, dimnames = list(alt.names, alt.names))
+    diag(Omega) <- pi^2 / 6
+  }
+  if (probit){
+    S <- matrix(0, J - 1, J - 1)
+    S[!upper.tri(S)] <- x$coefficients[-c(1:K)]
+    Mi <- list()
+    M <- rbind(0, diag(J - 2))
+    Mi[[1]] <- diag(J - 1)
+    for (i in 2:(J - 1)){
+      Mi[[i]] <- cbind(M[, 0:(i - 2), drop = FALSE], -1, M[, ((i - 1):(J - 2))])
+    }
+    Mi[[J]] <- cbind(M, -1)
+    names(Mi) <- alt.lev
+    Omega <- lapply(Mi, function(x) tcrossprod(x %*% S))
+    for (i in 1:J){
+      colnames(Omega[[i]]) <- rownames(Omega[[i]]) <- alt.lev[-i]
+    }
+  }
+  if (nested.logit | pair.comb.logit){
+    alt.names <- colnames(probabilities)
+    J <- length(alt.names)
+    Omega <- matrix(0, J, J, dimnames = list(alt.names, alt.names))
+    for (i in 1:length(nests)){
+      anEl <- x$coefficients[paste('iv', names(nests)[i], sep = ".")]
+      alts <- nests[[i]]
+      M <- length(alts)
+      for (m in 1:M){
+        for (n in 1:M){
+          Omega[alts[m], alts[n]] <- (1 - anEl^2)
+        }
+      }
+    }
+    diag(Omega) <- 1
+    Omega <- Omega * tcrossprod(rep(pi/sqrt(6), J))
+  }
+  if (heterosc.logit){
+    alt.names <- colnames(probabilities)
+    J <- length(alt.names)
+    Omega <- matrix(0, J, J, dimnames = list(alt.names, alt.names))
+    diag(Omega) <- pi^2 / 6
+    for (i in 2:J){
+      analt <- alt.names[i]
+      Omega[analt, analt] <- pi^2 / 6 * x$coefficients[paste('sp', analt, sep = '.')]^2
+    }
+  }
 
-  thecoef <- x$coefficients
-  attr(thecoef, "fixed") <- attr(x$optimum, "fixed")
   result <- structure(
                       list(
-                           coefficients  = thecoef,
+                           coefficients  = x$coefficients,
                            logLik        = logLik,
-                           gradient      = - attr(x$optimum, 'gradient'),
-                           gradi         = - attr(x$optimum, 'gradi'),
+                           gradient      = gradient,
                            hessian       = hessian,
                            est.stat      = x$est.stat,
-                           fitted.values = fit,
+                           fitted.values = fitted,
+                           probabilities = probabilities,
                            residuals     = resid,
+                           omega         = Omega,
+                           rpar          = rpar,
+                           nests         = nests,
                            model         = mf,
                            freq          = freq,
                            formula       = formula,
                            call          = callT),
                       class = 'mlogit'
-                      )
-  if (mixed.logit) result$rpar <- rpar
-  if (nested.logit) result$nests <- nests
+                      ) 
+  #result$Mi <- Mi
+  #result$alt.lev <- alt.lev
   result
 }
-
